@@ -25,6 +25,7 @@
 #include "php.h"
 #include "fopen_wrappers.h"
 #include "php_globals.h"
+#include "ext/standard/php_math.h"
 
 #include <stdlib.h>
 
@@ -302,8 +303,24 @@ PHP_FUNCTION(dbase_add_record)
 		
 		tmp = **field;
 		zval_copy_ctor(&tmp);
-		convert_to_string(&tmp);
-		snprintf(t_cp, cur_f->db_flen+1, cur_f->db_format, Z_STRVAL(tmp));
+		if (Z_TYPE(tmp) == IS_DOUBLE) {
+			char *formatted;
+			size_t formatted_len;
+
+			formatted = _php_math_number_format_ex(Z_DVAL_P(&tmp), cur_f->db_fdc, ".", 1, "", 0);
+			formatted_len = strlen(formatted);
+			if (formatted_len <= cur_f->db_flen) {
+				size_t delta = cur_f->db_flen - formatted_len;
+				memset(t_cp, ' ', delta);
+				memcpy(t_cp + delta, formatted, formatted_len);
+			} else {
+				memcpy(t_cp, formatted, cur_f->db_flen);
+			}
+			efree(formatted);
+		} else {
+			convert_to_string(&tmp);
+			snprintf(t_cp, cur_f->db_flen+1, cur_f->db_format, Z_STRVAL(tmp));
+		}
 		zval_dtor(&tmp); 
 		t_cp += cur_f->db_flen;
 	}
@@ -370,8 +387,24 @@ PHP_FUNCTION(dbase_replace_record)
 			efree(cp);
 			RETURN_FALSE;
 		}
-		convert_to_string_ex(field);
-		snprintf(t_cp, cur_f->db_flen+1, cur_f->db_format, Z_STRVAL_PP(field)); 
+		if (Z_TYPE_PP(field) == IS_DOUBLE) {
+			char *formatted;
+			size_t formatted_len;
+
+			formatted = _php_math_number_format_ex(Z_DVAL_PP(field), cur_f->db_fdc, ".", 1, "", 0);
+			formatted_len = strlen(formatted);
+			if (formatted_len <= cur_f->db_flen) {
+				size_t delta = cur_f->db_flen - formatted_len;
+				memset(t_cp, ' ', delta);
+				memcpy(t_cp + delta, formatted, formatted_len);
+			} else {
+				memcpy(t_cp, formatted, cur_f->db_flen);
+			}
+			efree(formatted);
+		} else {
+			convert_to_string_ex(field);
+			snprintf(t_cp, cur_f->db_flen+1, cur_f->db_format, Z_STRVAL_PP(field));
+		}
 		t_cp += cur_f->db_flen;
 	}
 
@@ -502,17 +535,17 @@ static void php_dbase_get_record(INTERNAL_FUNCTION_PARAMETERS, int assoc)
 					errno = errno_save;
 				} else {
 					if (!assoc) {
-						add_next_index_double(return_value, atof(str_value));
+						add_next_index_double(return_value, zend_strtod(str_value, NULL));
 					} else {
-						add_assoc_double(return_value, cur_f->db_fname, atof(str_value));
+						add_assoc_double(return_value, cur_f->db_fname, zend_strtod(str_value, NULL));
 					}
 				}
 				break;
 			case 'F':
 				if (!assoc) {
-					add_next_index_double(return_value, atof(str_value));
+					add_next_index_double(return_value, zend_strtod(str_value, NULL));
 				} else {
-					add_assoc_double(return_value, cur_f->db_fname, atof(str_value));
+					add_assoc_double(return_value, cur_f->db_fname, zend_strtod(str_value, NULL));
 				}
 				break;
 			case 'L':	/* we used to FALL THROUGH, but now we check for T/Y and F/N
@@ -616,6 +649,7 @@ PHP_FUNCTION(dbase_create)
 
 	if (num_fields <= 0) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to create database without fields");
+		close(fd);
 		RETURN_FALSE;
 	}
 
@@ -625,6 +659,7 @@ PHP_FUNCTION(dbase_create)
 	dbf = (dbfield_t *)malloc(sizeof(dbfield_t) * num_fields);
 	if (!dbh || !dbf) {
 		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to allocate memory for header info");
+		close(fd);
 		RETURN_FALSE;
 	}
 	
@@ -657,6 +692,7 @@ PHP_FUNCTION(dbase_create)
 		if (zend_hash_index_find(Z_ARRVAL_PP(fields), i, (void **)&field) == FAILURE) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "unable to find field %d", i);
 			free_dbf_head(dbh);
+			close(fd);
 			RETURN_FALSE;
 		}
 
@@ -664,12 +700,14 @@ PHP_FUNCTION(dbase_create)
 		if (Z_TYPE_PP(field) != IS_ARRAY || zend_hash_index_find(Z_ARRVAL_PP(field), 0, (void **)&value) == FAILURE) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "expected field name as first element of list in field %d", i);
 			free_dbf_head(dbh);
+			close(fd);
 			RETURN_FALSE;
 		}
 		convert_to_string_ex(value);
 		if (Z_STRLEN_PP(value) > 10 || Z_STRLEN_PP(value) == 0) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "invalid field name '%s' (must be non-empty and less than or equal to 10 characters)", Z_STRVAL_PP(value));
 			free_dbf_head(dbh);
+			close(fd);
 			RETURN_FALSE;
 		}
 		copy_crimp(cur_f->db_fname, Z_STRVAL_PP(value), Z_STRLEN_PP(value));
@@ -677,6 +715,7 @@ PHP_FUNCTION(dbase_create)
 		/* field type */
 		if (zend_hash_index_find(Z_ARRVAL_PP (field), 1, (void **)&value) == FAILURE) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "expected field type as second element of list in field %d", i);
+			close(fd);
 			RETURN_FALSE;
 		}
 		convert_to_string_ex(value);
@@ -706,6 +745,7 @@ PHP_FUNCTION(dbase_create)
 			if (zend_hash_index_find(Z_ARRVAL_PP (field), 2, (void **)&value) == FAILURE) {
 				php_error_docref(NULL TSRMLS_CC, E_WARNING, "expected field length as third element of list in field %d", i);
 				free_dbf_head(dbh);
+				close(fd);
 				RETURN_FALSE;
 			}
 			convert_to_long_ex(value);
@@ -715,6 +755,7 @@ PHP_FUNCTION(dbase_create)
 				if (zend_hash_index_find(Z_ARRVAL_PP (field), 3, (void **)&value) == FAILURE) {
 					php_error_docref(NULL TSRMLS_CC, E_WARNING, "expected field precision as fourth element of list in field %d", i);
 					free_dbf_head(dbh);
+					close(fd);
 					RETURN_FALSE;
 				}
 				convert_to_long_ex(value);
@@ -724,6 +765,7 @@ PHP_FUNCTION(dbase_create)
 		default:
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "unknown field type '%c'", cur_f->db_type);
 			free_dbf_head(dbh);
+			close(fd);
 			RETURN_FALSE;
 		}
 		cur_f->db_foffset = rlen;
